package org.openstreetmap.gui.jmapviewer.interfaces;

public interface ICoordinate {
    public double getLat();
    public void setLat(double lat);
    public double getLon();
    public void setLon(double lon);
}
